/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.designer.models.transformers.bpmn2designer;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;
import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.PoolModel;
import org.ow2.orchestra.designer.bpmn.model.TaskModel;
import org.ow2.orchestra.designer.bpmn.model.TaskType;
import org.ow2.orchestra.designer.bpmn.model.operations.InterfaceModel;
import org.ow2.orchestra.designer.bpmn.model.operations.OperationModel;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.jaxb.bpmn.Assignment;
import org.ow2.orchestra.jaxb.bpmn.DataInputAssociation;
import org.ow2.orchestra.jaxb.bpmn.DataOutputAssociation;
import org.ow2.orchestra.jaxb.bpmn.Operation;
import org.ow2.orchestra.jaxb.bpmn.TDataObject;
import org.ow2.orchestra.jaxb.bpmn.TInterface;
import org.ow2.orchestra.jaxb.bpmn.TRootElement;
import org.ow2.orchestra.jaxb.bpmn.TServiceTask;
import org.ow2.orchestra.jaxb.bpmn.TTask;

/**
 * Transform {@link TTask}s into {@link TaskModel}s
 *
 * @author Guillaume Porcher
 */
public final class TaskTransformer {

  /**
   * Utility classes should not have public or default constructors.
   */
  private TaskTransformer() {

  }


  public static void bpmnToModel(final TTask tTask, final Bpmn2DesignerContext context) throws TransformationException {
    final TaskModel taskModel = new TaskModel();
    taskModel.setProcessModel(context.getProcessModel());
    taskModel.setName(tTask.getName());
    if (tTask instanceof TServiceTask) {
      taskModel.setTaskType(TaskType.SERVICE);

      // Get operation name and interface name
      final TServiceTask serviceTask = (TServiceTask) tTask;
      final QName operationQName = serviceTask.getOperationRef();
      if (operationQName != null && operationQName.getNamespaceURI() != null && !"".equals(operationQName.getNamespaceURI())) {
        if (!operationQName.getNamespaceURI().equals(context.getDefinitions().getTargetNamespace())) {
          throw new TransformationException("BPMN conversion error: task " + serviceTask.getId() + " references an unknown operation.");
        }
        TInterface tInterface = null;
        Operation operation = null;
        for (final JAXBElement<? extends TRootElement> rootElement : context.getDefinitions().getRootElements()) {
          if (rootElement.getValue() instanceof TInterface) {
            for (final Operation interfaceOperation : ((TInterface) rootElement.getValue()).getOperations()) {
              if (interfaceOperation.getId().equals(operationQName.getLocalPart())) {
                tInterface = (TInterface) rootElement.getValue();
                operation = interfaceOperation;
              }
            }
          }
        }
        if (operation == null) {
          throw new TransformationException("BPMN conversion error: task " + serviceTask.getId() + " references an unknown operation.");
        }
        InterfaceModel interfaceModel = null;
        for (AbstractElement elt : context.getProcessModel().getElements()) {
          if (elt instanceof PoolModel) {
            PoolModel poolModel = (PoolModel) elt;
            poolModel.getInterfaces().containsKey(tInterface.getName());
            interfaceModel = poolModel.getInterfaces().get(tInterface.getName());
            break;
          }
        }
        if (interfaceModel == null) {
          throw new TransformationException("BPMN conversion error: task " + serviceTask.getId() + " references an unknown operation.");
        }
        final OperationModel operationModel = interfaceModel.getOperations().get(operation.getName());
        if (operationModel == null) {
          throw new TransformationException("BPMN conversion error: task " + serviceTask.getId() + " references an unknown operation.");
        }
        taskModel.setOperation(operationModel);
      }
      // Get expression
      String expression = null;
      for (final DataInputAssociation dataInputAssociation : tTask.getDataInputAssociations()) {
        for (final Assignment assignment : dataInputAssociation.getAssignments()) {
          for (final Object content : assignment.getFrom().getContent()) {
            if (content instanceof String && !"".equals(content)) {
              expression = (String) content;
            }
          }
        }
      }
      taskModel.setInputExpression(expression);
      // Get variable name
      String variableName = null;
      for (final DataOutputAssociation dataOutputAssociation : tTask.getDataOutputAssociations()) {
        if (dataOutputAssociation.getTargetRef() instanceof TDataObject) {
          variableName = ((TDataObject) dataOutputAssociation.getTargetRef()).getName();
        }
      }
      taskModel.setOutputVariableName(variableName);
    }
    context.getProcessModel().getElements().add(taskModel);
    context.getModelsMap().put(tTask, taskModel);


    if (tTask.getId() == null || "".equals(tTask.getId())) {
      // If there is no id, we will not be able to find out a diagram element
      // for this gateway.
      // We will not be able to reference this gateway from the designer, so this is an error
      throw new TransformationException("BPMN conversion error: A task has no id.");
    }
    taskModel.setId(tTask.getId());
    // Now retrieve positions
    ShapeTransformer.bpmnToModel(tTask.getId(), taskModel, context);
  }

}
